
/*
	lkmonitor (Linux Kernel Monitor)

	Application for monitoring and tuning a Linux kernel.

	Copyright (C) 2005-2008  Fernando Apesteguía

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include <glib.h>

#define PROC_DIR  	"/proc/"
#define CPU_FILE  	"/proc/cpuinfo"
#define CMD_FILE "/proc/cmdline"
#define VER_FILE "/proc/version"
#define LOAD_FILE "/proc/loadavg"
#define IOMEM_FILE "/proc/iomem"
#define IOPORTS_FILE "/proc/ioports"
#define DMA_FILE "/proc/dma"
#define MEM_FILE  	"/proc/meminfo"
#define DEV_FILE  	"/proc/devices"
#define FILES	  	"/proc/sys/fs/file-nr"
#define FILE_MAX	"/proc/sys/fs/file-max"
#define INODES		"/proc/sys/fs/inode-nr"
#define HOST_NAME 	"/proc/sys/kernel/hostname"
#define DOMAIN_NAME	"/proc/sys/kernel/domainname"
#define P_MAX_Q		"/proc/sys/fs/mqueue/queues_max"
#define P_MAX_M		"/proc/sys/fs/mqueue/msg_max"
#define P_MAX_S		"/proc/sys/fs/mqueue/msgsize_max"
#define PAGE_MAX	"/proc/sys/kernel/shmall"
#define	SEG_MAX		"/proc/sys/kernel/shmmni"
#define	SEG_SIZE	"/proc/sys/kernel/shmmax"
#define	THREADS_MAX	"/proc/sys/kernel/threads-max"
#define SEM			"/proc/sys/kernel/sem"
#define STATS_FILE 	"/proc/stat"
#define BATT_STATE	"/proc/acpi/battery/BAT0/state"
#define BATT_INFO	"/proc/acpi/battery/BAT0/info"
#define BATT_STATE1	"/proc/acpi/battery/BAT1/state"
#define BATT_INFO1	"/proc/acpi/battery/BAT1/info"
#define TEMPERATURE "/proc/acpi/thermal_zone/THRM/temperature"
#define TEMP_TRIP "/proc/acpi/thermal_zone/THRM/trip_points"
#define NET_DEVS "/proc/net/dev"
#define NET_PROTOCOLS	"/proc/net/protocols"
#define FS_FILE  "/proc/filesystems"
#define MOD_FILE	"/proc/modules"

#define MAX_FIELD_SZ  255
#define FIELD_SZ  15


#define TV_DEVS	0
#define TV_MODS	1

/*!\struct modules_info lininfo.h
 *\brief Contains modules information
 * This struct contains a dumpe of the /proc/modules file
*/
struct modules_info{
  GString *contents;
};

/*!\struct fs_info lininfo.h
 *\brief Filesystem information
 * This structure keeps information about filesystems loaded in the kernel.
 * The filesystems are splitted in two categories depending on if they are
 * attached to any devices or not.
*/
struct fs_info{
  GString *devs;
  GString *nodevs;
};

/*!\struct net_dev_info lininfo.h
 *\brief Contains statistics for net usage with a certain device
 * Contains information about usage of this net device.
 * When the field is an array, the first element [0] if for received
 * and the second one [1] for transmitted elements.
*/
struct net_dev_info{
	unsigned long long int bytes[2];		///< Bytes received/transmitted
	unsigned long int packets[2];	///< Packets received/transmitted
	int errs[2];		///< Errors
	int drop[2];		///< No space in linux buffers
	int fifo[2];		///< Received/trasnmited FIFO overruns
	int r_frame;		///< Received frame alignment error
	int t_colls;		///< Collisions
	int compressed[2];///< Compressed cslip received/transmitted
	int r_multicast;	///< Multicast packets received
	int t_carrier;		///< Carrier errors in transmission
};

/*! \struct settings_info lininfo.h
*   \brief Contains system settings.
*  This struct contains several fields that represent different system
*  settings like hostname maximum for a shared memory segment, etc.
*/
struct settings_info
{
	int fdmax;		///< Max. File descriptors
	int fdasig;		///< Asigned file descriptors
	int fdfreed;	///< Freed file descriptors
	int inmax;		///< Max. inodes
	int inasig;		///< Asigned inodes
	int posix_mq;	///< POSIX max. queues
	int posix_mpq;	///< POSIX max. messages per queue
	int posix_ms;	///< POSIX max. message size
	GString *host;	///< Hostname
	GString *domain;///< Domain name	
	int page_max;	///< Max. shared memory pages
	int seg_max;	///< Max. shared memory segments
	int seg_size;	///< Max. shared memory segment size
	int max_seminset;///< Max. semaphores in a set
	int max_semsys;	///<  Max. semaphores in system
	int max_opcall;	///<  Max. operations per call
	int max_semsets;///<  Max. semaphore sets.
	int threads;	///< Max. threads in system
};

/*! \struct dev_info lininfo.h
*   \brief Contains device lists.
*  This struct contains two device lists.
*/
struct dev_info
{
	GString *characters;	///< Character devices
	GString *blocks;		///< Block devices
};

/*! \struct cpu_info lininfo.h
*   \brief Contains cpu characteristics.
*  This struct contains several fields that represent different cpu
*  characteristics as clock MHz or cache size
*/
struct cpu_info
{
  char processor[FIELD_SZ];	///< Id of the processor
  char vendor_id[2 * FIELD_SZ];	///< Vendor Id
  char cpu_family[2 * FIELD_SZ];///< Cpu family (currently an integer number)
  char model[FIELD_SZ];		///<Integer id. for the model
  char model_name[5 * FIELD_SZ];	///<Vendor's name for the model
  char stepping[FIELD_SZ];///< Microprocessor's stepping
  char Mhz[FIELD_SZ];///< Cpu clock
  char cache[FIELD_SZ];///< Second level cache size
  char fpu[FIELD_SZ / 2];///< Float Point Unit
  char fpu_exception[FIELD_SZ / 2];///< Cpu uses specific fpu exception
  char cpuid_level[FIELD_SZ / 2];///< For SMP	
  char wp[FIELD_SZ / 2];	///< Wether the wp bit is present
  char flags[50 * FIELD_SZ];///< Specific characteristics for this model
  char bogomips[FIELD_SZ];///< Speed unit used in Linux.
};

/*! \struct mem_info lininfo.h
*   \brief Contains memory data and statistics.
*   Contains fields for total, free and other interesting information
*/
struct mem_info
{
  char total[2*FIELD_SZ];	///< Total physical memory
  char free[FIELD_SZ];	///< Free memory
  char shared[FIELD_SZ]; ///< Shared
  char buffers[FIELD_SZ]; ///< Kb used for buffers
  char cached[FIELD_SZ];  ///< Kb cached
  char swap_cached[FIELD_SZ]; ///< Kb cached for swap memory
  char active[FIELD_SZ];  ///< Kb of active memory
  char inactive[FIELD_SZ]; ///< Kb of inactive memory
  char dirty[FIELD_SZ];  ///< Kb of dirty memory
  char high_total[FIELD_SZ];  ///< Total high memory
  char high_free[FIELD_SZ];  ///< Free memory 
  char low_total[2*FIELD_SZ];  ///< Total low memory 
  char low_free[FIELD_SZ];   ///< Free low memory
  char swap_total[FIELD_SZ]; ///< Kb of swap 
  char swap_free[FIELD_SZ];   ///< Kb free swap
  char hugepages_total[FIELD_SZ]; ///< Number of hugepage
  char hugepages_free[FIELD_SZ];  ///< Free hupages
  char hugepages_size[FIELD_SZ];  ///< Size of each hupage
};

/*! \struct stats_info lininfo.h
*   \brief Contains statistics about cpu time usage.
*   Contains fields for global and per cpu/core statistics. Context switches
*	is also provided
*/
struct stats_info
{
  GString *cpu;		///< Global count
  GString *cpu0;	///< Count for the first cpu/core
  GString *cpu1;	///< Count for the second cpu/core
  GString *ctxt;	///< Context switches
};

/*! \struct battery_info lininfo.h
*   \brief Battery status information.
*   Contains fields for charge and capacity data.
*	Referenced by acpi_info structure
*/
struct battery_info
{
	GString *rem;		///< Capacity remaining
	GString *cap;		///< Design capacity
	GString *last_cap;	///< Last full capacity
	GString *type;		///< Battery type
	GString *cap_warning;	///< Capacity warning treshold
	GString *cap_low;	///< Capacity low treshold
};


/*! \struct acpi_info lininfo.h
*   \brief Contains temperature and battery statistics.
*   Uses the battery_info structure
*/
struct acpi_info
{
	GString *temp;			///< Current temperature
	GString *temp_critical;		///< Critical temperature
	struct battery_info battery;	///< Battery info (battery_info struct)
};

/*! \struct cmd_ver lininfo.h
*   \brief Contains information for cmdline, system version and load average
*   This structure is used to store cmdline information (like kernel parameters)
*   the system version (kernel, gcc..) and the load average.
*/
struct cmd_ver
{
  GString *cmd;
  GString *version;
  GString *loadavg;
};


/*! \struct channels lininfo.h
*  \brief Struct with information about communication channels
*  This structure keeps information about communication channels
*  like IO ports, IO mem and DMA.
*/
struct channels_info{
  GString *ioports;
  GString *iomem;
  GString *dma;	
};

int get_cpu_info ();		///< Fills cpu_info struct fields
int get_mhz_info();		///< Updates Mhz (for Centrino, AMD64...)
int get_mem_info ();		///< Fills mem_struct fields
int get_devs_info();		///< Fills devs_struct fields
int get_settings_info();	///< Get settings values
int get_stats_info ();		///< Get cpu times
int get_acpi_info();		///< Collects ACPI data
int get_battery_info();  	///< Get static information about the battery
int get_cmd_ver_info(); 	///< Get cmdline and version kernel information
int get_loadavg_info();		///< Get load average information
int get_channels_info();	///< Get infor for communication channels
int get_net_info();		///< Get info for net devices
int get_fs_info();		///< Get info about filesystems
int get_mod_info();
